unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, pngimage, MiTeC_WeatherForecast;

type
  Twnd_Weather = class(TForm)
    eLoc: TEdit;
    LocList: TListView;
    WFList: TListView;
    Panel1: TPanel;                               
    lDesc: TLabel;
    lWind: TLabel;
    lCloud: TLabel;
    lPrec: TLabel;
    lLoc: TLabel;
    lHum: TLabel;
    Panel2: TPanel;
    Image: TImage;
    lTemp: TLabel;
    Panel3: TPanel;
    OWM: TImage;
    lPress: TLabel;
    eKey: TEdit;
    Label2: TLabel;
    Label1: TLabel;
    cbLang: TComboBox;
    Label3: TLabel;
    cbUnits: TComboBox;
    Button1: TButton;
    WeatherForecast: TWeatherForecast;
    Label4: TLabel;
    cbMode: TComboBox;
    procedure eLocKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure LocListDblClick(Sender: TObject);
    procedure LocListKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OWMClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    sl: TStringList;
    procedure RefreshLocations;
    procedure RefreshWeather;
  public
  end;

var
  wnd_Weather: Twnd_Weather;

implementation

uses ShellAPI, MiTeC_OWM;

{$R *.dfm}

procedure Twnd_Weather.Button1Click(Sender: TObject);
begin
  Shellexecute(Handle,'open','http://openweathermap.org/appid#get',nil,nil,SW_SHOWDEFAULT);
end;

procedure Twnd_Weather.eLocKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Shift=[]) and (Key=vk_return) then begin
    WeatherForecast.Active:=True;
    RefreshLocations;
  end;
end;

procedure Twnd_Weather.FormCreate(Sender: TObject);
begin
  sl:=tstringlist.Create;
end;

procedure Twnd_Weather.LocListDblClick(Sender: TObject);
begin
  if not Assigned(LocList.Selected) then
    Exit;
  RefreshWeather;
end;

procedure Twnd_Weather.LocListKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift=[]) and (Key=vk_return) then
    RefreshWeather;
end;

procedure Twnd_Weather.OWMClick(Sender: TObject);
begin
  Shellexecute(Handle,'open','http://openweathermap.org/',nil,nil,SW_SHOWDEFAULT);
end;

procedure Twnd_Weather.RefreshLocations;
var
  i: Integer;
begin
  Screen.Cursor:=crHourglass;
  LocList.Items.Clear;
  try
    WeatherForecast.Location:=eLoc.Text;
    WeatherForecast.APIKey:=eKey.Text;
    WeatherForecast.Language:=TLanguage(cbLang.ItemIndex);
    WeatherForecast.Units:=TUnits(cbUnits.ItemIndex);
    WeatherForecast.ForecastMode:=TForecastMode(cbMode.ItemIndex);
    if not WeatherForecast.FindLocations then begin
      if WeatherForecast.GetLastError<>'' then
        MessageDlg(WeatherForecast.GetLastError,mtError,[mbOK],0);
      Exit;
    end;
    for i:=0 to WeatherForecast.LocationCount-1 do
      with LocList.Items.Add do begin
        Caption:=WeatherForecast.Locations[i].Name;
        SubItems.Add(WeatherForecast.Locations[i].Country);
        SubItems.Add(FloatToStr(WeatherForecast.Locations[i].Lat));
        SubItems.Add(FloatToStr(WeatherForecast.Locations[i].Lon));
        SubItems.Add(DatetimeToStr(WeatherForecast.Locations[i].Sunrise));
        SubItems.Add(DatetimeToStr(WeatherForecast.Locations[i].Sunset));
      end;
    if LocList.Items.Count>0 then
      LocList.Selected:=LocList.Items[0];
  finally
    Screen.Cursor:=crDefault;
  end;
  LocList.SetFocus;

  if LocList.Items.Count=1 then
    RefreshWeather;
end;

procedure Twnd_Weather.RefreshWeather;
var
  i,c: Integer;
begin
  Screen.Cursor:=crHourglass;
  WFList.Items.Clear;
  try
    if not Assigned(LocList.Selected) then
      Exit;
    lLoc.Caption:=Format('%s, %s',[LocList.Selected.Caption,LocList.Selected.SubItems[0]]);
    WeatherForecast.APIKey:=eKey.Text;
    WeatherForecast.Language:=TLanguage(cbLang.ItemIndex);
    WeatherForecast.Units:=TUnits(cbUnits.ItemIndex);
    WeatherForecast.Location:=lLoc.Caption;
    {$B+}
    if (WeatherForecast.GetLastError<>'') and not WeatherForecast.RefreshForecast then begin
      MessageDlg(WeatherForecast.GetLastError,mtError,[mbOK],0);
      Exit;
    end;
    {$B-}
    WeatherForecast.DownloadIcons(ExtractFilePath(Application.ExeName));

    with WeatherForecast.CurrentWeather do begin
      if FileExists(WeatherIcon) then
        Image.Picture.LoadFromFile(WeatherIcon);
      lDesc.Caption:=Text;
      lTemp.Caption:=Temperature.Text[1];
      lPress.Caption:=Pressure.Text[1];
      lWind.Caption:=Format('Wind: %s %s (%s)',[WindSpeed.Text[2],WindDirection,WindText]);
      lPrec.Caption:=Format('Precipitations: %s',[Precipitations.Text[2]]);
      lHum.Caption:=Format('Humidity: %s',[Humidity.Text[0]]);
      lCloud.Caption:=Format('Cloudiness: %s',[Cloudiness.Text[0]]);
    end;

    case WeatherForecast.ForecastMode of
      fmHourly: c:=WeatherForecast.NumberOfDays*8-1;
      fmDaily: c:=WeatherForecast.NumberOfDays-1;
    end;

    for i:=0 to c do
      with WeatherForecast.Forecast[i],WFList.Items.Add do begin
        case WeatherForecast.ForecastMode of
          fmHourly: Caption:=DateTimeToStr(FromTime)+' - '+DateTimeToStr(ToTime);
          fmDaily: Caption:=DateToStr(SysUtils.Date+i);
        end;
        SubItems.Add(Format('%s, %s',[Text,WindText]));
        SubItems.Add(Format('%1.0f/%1.0f %s',[Temperature.Max,Temperature.Min,Temperature.Units]));
        SubItems.Add(Format('%s %s',[WindSpeed.Text[2],WindDirection]));
        SubItems.Add(Precipitations.Text[2]);
        SubItems.Add(Pressure.Text[1]);
        SubItems.Add(Humidity.Text[0]);
        SubItems.Add(Cloudiness.Text[0]);
      end;
  finally
    Screen.Cursor:=crDefault;
  end;
  eLoc.SetFocus;
end;

end.
